=== modified file 'NEWS'
--- NEWS	2006-03-22 22:37:54 +0000
+++ NEWS	2006-04-12 01:27:15 +0000
@@ -1,3 +1,6 @@
+Apr 11
+* New Switch command (David Allouche/Canonical)
+
 Mar 22
 * New Zap command
 

=== modified file 'switch.py'
--- switch.py	2006-04-12 00:28:48 +0000
+++ switch.py	2006-04-12 01:27:15 +0000
@@ -1,3 +1,20 @@
+# Copyright (C) 2006 Canonical Limited.
+# Authors: David Allouche <david@allouche.net>
+#          Aaron Bentley <aaron.bentley@utoronto.ca>
+#
+#    This program is free software; you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation; either version 2 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
+#
+#    You should have received a copy of the GNU General Public License
+#    along with this program; if not, write to the Free Software
+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 from bzrlib.errors import BzrCommandError
 from bzrlib.commands import Command, register_command
 from bzrlib.branch import Branch, BranchFormat
@@ -5,6 +22,7 @@
 from bzrlib.transport import get_transport
 from bzrlib.builtins import cmd_update
 from bzrlib.trace import note
+from bzrlib.workingtree import WorkingTree
 
 class cmd_switch(Command):
     """Set the branch of a lightweight checkout and update."""
@@ -14,16 +32,23 @@
     def run(self, to_location):
         to_branch = Branch.open(to_location)
         url = '.'
-        branch = Branch.open_containing(url)[0]
-        branch.lock_write()
+        wt = WorkingTree.open_containing(url)[0]
+        branch = wt.branch
+        unlockables = []
         try:
-            control = BzrDir.open_containing(url)[0]
+            wt.lock_write()
+            unlockables.append(wt)
+            branch.lock_write()
+            unlockables.append(branch)
             self._check_switch_branch_format(url)
-            self.set_branch_location(control, to_branch.base)
+            self.set_branch_location(wt.bzrdir, to_branch.base)
+            note('Switched to branch: %s' % (to_branch.base,))
+            wt.update()
+            note('Updated to revision %d' %
+                 (wt.branch.revision_id_to_revno(wt.last_revision()),))
         finally:
-            branch.unlock()
-        note('Switched to branch: %s' % (to_branch.base,))
-        cmd_update().run(url)
+            for unlockable in unlockables:
+                unlockable.unlock()
 
     def _check_switch_branch_format(self, url):
         transport = get_transport(url)

